/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "SignalMenu.h"

#include "CppUtil.h"
#include "Debug.h"
#include "IconEngine.h"
#include "IconNames.h"

#include <QActionGroup>
#include <sys/types.h>
#include <signal.h>

//________________________________________________________________
SignalMenu::SignalMenu( QWidget* parent ):
    QMenu( parent ),
    Counter( "SignalMenu" )
{
    Debug::Throw( "SignalMenu::SignalMenu.\n" );

    setTitle( tr( "Send Signal" ) );

    actions_ = Base::makeT<ActionMap>(
    {
        { addAction( IconEngine::get( IconNames::Pause ), signalName(SIGSTOP) ), SIGSTOP },
        { addAction( IconEngine::get( IconNames::Resume ), signalName(SIGCONT) ), SIGCONT },
        { addAction( signalName(SIGINT) ), SIGINT },
        { addAction( signalName(SIGTERM) ), SIGTERM },
        { addAction( IconEngine::get( IconNames::Kill ), signalName(SIGKILL) ), SIGKILL }
    });

    // action group
    QActionGroup* group = new QActionGroup( this );
    group->setExclusive( false );
    connect( group, SIGNAL(triggered(QAction*)), SLOT(_requestSignal(QAction*)) );

    for( auto&& iter = actions_.begin(); iter != actions_.end(); ++iter )
    { group->addAction( iter.key() ); }

}

//________________________________________________________________
QString SignalMenu::signalName( int signal ) const
{
    Debug::Throw( "SignalMenu:signalName.\n" );
    switch( signal )
    {
        case SIGSTOP: return tr( "Suspend" );
        case SIGCONT: return tr( "Resume" );
        case SIGINT: return tr( "Interrupt" );
        case SIGTERM: return tr( "Terminate" );
        case SIGKILL: return tr( "Kill" );
        default: return tr( "Unknown Signal" );
    }

}

//________________________________________________________________
void SignalMenu::_requestSignal( QAction* action )
{
    Debug::Throw( "SignalMenu:_requestSignal.\n" );
    ActionMap::const_iterator iter( actions_.find( action ) );
    if( iter != actions_.end() ) emit signalRequested( iter.value() );
}
